/*
 * Copyright (c) 2019-2020 Arm Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * \file syswdog_armv8-m_drv.c
 *
 * \brief Driver for Armv8-M System Watchdog
 *
 */

#include <stdint.h>
#include <stdbool.h>
#include "syswdog_armv8-m_drv.h"

/** Setter bit manipulation macro */
#define SET_BIT(WORD, BIT_INDEX) ((WORD) |= (1u << (BIT_INDEX)))
/** Clearing bit manipulation macro */
#define CLR_BIT(WORD, BIT_INDEX) ((WORD) &= ~(1u << (BIT_INDEX)))
/** Getter bit manipulation macro */
#define GET_BIT(WORD, BIT_INDEX) (bool)(((WORD) & (1u << (BIT_INDEX))))
/** Getter bit-field manipulation macro */
/** Bit mask for given width bit-field manipulation macro */
#define BITMASK(width) ((1u<<(width))-1)
#define GET_BIT_FIELD(WORD, WIDTH, BIT_OFFSET) \
    ((WORD & ((BITMASK(WIDTH)) << BIT_OFFSET)) >> BIT_OFFSET)

/**
 * \brief Watchdog Control and Refresh Frame register map structure
 */
struct wdog_frame_reg_map_t {
/* Control Frame registers */
    volatile uint32_t wcs;
        /*!< Offset: 0x0000 (RW) Watchdog Control and Status register */
    volatile uint32_t reserved0;
        /*!< Offset: 0x0004 (RES0) Reserved */
    volatile uint32_t wor;
        /*!< Offset: 0x0008 (RW) Watchdog Offset register */
    volatile uint32_t reserved1;
        /*!< Offset: 0x000C (RES1) Reserved */
    volatile uint32_t wcv_low;
        /*!< Offset: 0x0010 (RW) Watchdog Compare Value low register [31:0] */
    volatile uint32_t wcv_high;
        /*!< Offset: 0x0014 (RW) Watchdog Compare Value high register [63:32] */
    volatile uint32_t reserved2[1005];
        /*!< Offset: 0x0018 (RES2) Reserved */
    volatile uint32_t cnt_w_iidr;
        /*!< Offset: 0xFCC (RO) Watchdog Interface Identification register */
    volatile uint32_t cnt_pidr4;
        /*!< Offset: 0xFD0 Peripheral ID 4 */
    volatile uint32_t reserved3[3];
        /*!< Offset: 0x0FD4 (RES3) Reserved */
    volatile uint32_t cnt_pidr0;
        /*!< Offset: 0xFE0 Peripheral ID 0 */
    volatile uint32_t cnt_pidr1;
        /*!< Offset: 0xFE4 Peripheral ID 1 */
    volatile uint32_t cnt_pidr2;
        /*!< Offset: 0xFE8 Peripheral ID 2 */
    volatile uint32_t cnt_pidr3;
        /*!< Offset: 0xFEC Peripheral ID 3 */
    volatile uint32_t cnt_cidr0;
        /*!< Offset: 0xFF0 Component ID 0 */
    volatile uint32_t cnt_cidr1;
        /*!< Offset: 0xFF4 Component ID 1 */
    volatile uint32_t cnt_cidr2;
        /*!< Offset: 0xFF8 Component ID 2 */
    volatile uint32_t cnt_cidr3;
        /*!< Offset: 0xFFC Component ID 3 */
/* Refresh Frame registers */
    volatile uint32_t wrr;
        /*!< Offset: 0x1000 (RW) Watchdog Refresh register */
    volatile uint32_t reserved4[1010];
        /*!< Offset: 0x1004 (RES4) Reserved */
    volatile uint32_t ref_w_iidr;
        /*!< Offset: 0x1FCC (RO) Watchdog Interface Identification register */
    volatile uint32_t ref_pidr4;
        /*!< Offset: 0x1FD0 Peripheral ID 4 */
    volatile uint32_t reserved5[3];
        /*!< Offset: 0x1FD4 (RES5) Reserved */
    volatile uint32_t ref_pidr0;
        /*!< Offset: 0x1FE0 Peripheral ID 0 */
    volatile uint32_t ref_pidr1;
        /*!< Offset: 0x1FE4 Peripheral ID 1 */
    volatile uint32_t ref_pidr2;
        /*!< Offset: 0x1FE8 Peripheral ID 2 */
    volatile uint32_t ref_pidr3;
        /*!< Offset: 0x1FEC Peripheral ID 3 */
    volatile uint32_t ref_cidr0;
        /*!< Offset: 0x1FF0 Component ID 0 */
    volatile uint32_t ref_cidr1;
        /*!< Offset: 0x1FF4 Component ID 1 */
    volatile uint32_t ref_cidr2;
        /*!< Offset: 0x1FF8 Component ID 2 */
    volatile uint32_t ref_cidr3;
        /*!< Offset: 0x1FFC Component ID 3 */
};

#define SYSWDOG_ARMV8_M_REGISTER_BIT_WIDTH          32u
    /*!< Armv8-M System Timer registers bit width */

/**
 * \brief Watchdog Control and Status register bit fields
 */
#define SYSWDOG_ARMV8M_CNTR_WCS_EN_OFF          0u
    /*!< Control and Status register Watchdog Enable bit field offset */
#define SYSWDOG_ARMV8M_CNTR_WCS_WS0_OFF         1u
    /*!< Control and Status register Watchdog Signal 0 bit field offset */
#define SYSWDOG_ARMV8M_CNTR_WCS_WS1_OFF         2u
    /*!< Control and Status register Watchdog Signal 1 bit field offset */

/**
 * \brief Watchdog Interface Identification register bit fields
 */
#define SYSWDOG_ARMV8M_W_IIDR_JEPCODE_OFF          0u
    /*!< Interface Identification register Arm JEP106 code bit field offset */
#define SYSWDOG_ARMV8M_W_IIDR_JEPCODE_SIZE         12u
    /*!< Interface Identification register Arm JEP106 code bit field size */
#define SYSWDOG_ARMV8M_W_IIDR_REV_OFF              12u
    /*!< Interface Identification register Revision number bit field offset */
#define SYSWDOG_ARMV8M_W_IIDR_REV_SIZE             4u
    /*!< Interface Identification register Revision number bit field size */
#define SYSWDOG_ARMV8M_W_IIDR_ARCH_OFF             16u
    /*!< Interface Identification register Architecture ver. bit field offset */
#define SYSWDOG_ARMV8M_W_IIDR_ARCH_SIZE            4u
    /*!< Interface Identification register Architecture ver. bit field size */
#define SYSWDOG_ARMV8M_W_IIDR_ID_OFF               24u
    /*!< Interface Identification register Product ID bit field offset */
#define SYSWDOG_ARMV8M_W_IIDR_ID_SIZE              8u
    /*!< Interface Identification register Product ID bit field size */


void syswdog_armv8_m_enable_wdog(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    SET_BIT(p_wdog->wcs, SYSWDOG_ARMV8M_CNTR_WCS_EN_OFF);
}

void syswdog_armv8_m_disable_wdog(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    CLR_BIT(p_wdog->wcs, SYSWDOG_ARMV8M_CNTR_WCS_EN_OFF);
}

bool syswdog_armv8_m_is_wdog_enabled(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT(p_wdog->wcs, SYSWDOG_ARMV8M_CNTR_WCS_EN_OFF);
}

bool syswdog_armv8_m_read_irq_status_0(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT(p_wdog->wcs, SYSWDOG_ARMV8M_CNTR_WCS_WS0_OFF);
}

bool syswdog_armv8_m_read_irq_status_1(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT(p_wdog->wcs, SYSWDOG_ARMV8M_CNTR_WCS_WS1_OFF);
}

void syswdog_armv8_m_set_offset(struct syswdog_armv8_m_dev_t* dev,
                                const uint32_t value)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    p_wdog->wor = value;
}

uint32_t syswdog_armv8_m_get_offset(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return p_wdog->wor;
}

void syswdog_armv8_m_set_compare_value(struct syswdog_armv8_m_dev_t* dev,
                                       const uint64_t value)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    p_wdog->wcv_high = (uint32_t)(value >> SYSWDOG_ARMV8_M_REGISTER_BIT_WIDTH);
    p_wdog->wcv_low = (uint32_t)(value &
                    (((uint64_t)1 << SYSWDOG_ARMV8_M_REGISTER_BIT_WIDTH) -1 ));
}

uint64_t syswdog_armv8_m_get_compare_value(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    uint64_t cmp_val = (uint64_t)p_wdog->wcv_high;
    cmp_val <<= SYSWDOG_ARMV8_M_REGISTER_BIT_WIDTH;
    cmp_val |= (uint64_t)p_wdog->wcv_low;

    return cmp_val;
}

void syswdog_armv8_m_refresh_wdog(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    /* Any write to refrsh reg will causes watchdog refresh. */
    p_wdog->wrr = 1;
}

uint32_t syswdog_armv8_m_get_cntr_product_id(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->cnt_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_ID_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_ID_OFF);
}

uint32_t syswdog_armv8_m_get_cntr_architecture_version(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->cnt_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_ARCH_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_ARCH_OFF);
}

uint32_t syswdog_armv8_m_get_cntr_revision_number(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->cnt_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_REV_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_REV_OFF);
}

uint32_t syswdog_armv8_m_get_cntr_arm_JEP106_code(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->cnt_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_JEPCODE_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_JEPCODE_OFF);
}

uint32_t syswdog_armv8_m_get_refr_product_id(struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->ref_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_ID_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_ID_OFF);
}

uint32_t syswdog_armv8_m_get_refr_architecture_version(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->ref_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_ARCH_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_ARCH_OFF);
}

uint32_t syswdog_armv8_m_get_refr_revision_number(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->ref_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_REV_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_REV_OFF);
}

uint32_t syswdog_armv8_m_get_refr_arm_JEP106_code(
                                struct syswdog_armv8_m_dev_t* dev)
{
    struct wdog_frame_reg_map_t* p_wdog =
                (struct wdog_frame_reg_map_t*)dev->cfg->base;

    return GET_BIT_FIELD(p_wdog->ref_w_iidr,
                         SYSWDOG_ARMV8M_W_IIDR_JEPCODE_SIZE,
                         SYSWDOG_ARMV8M_W_IIDR_JEPCODE_OFF);
}

void syswdog_armv8_m_init(struct syswdog_armv8_m_dev_t* dev, uint32_t offset)
{
    syswdog_armv8_m_set_offset(dev, offset);
    syswdog_armv8_m_enable_wdog(dev);
}
